@@ -335,7 +335,7 @@ sub find_local_mirror {
     my ($repos, $uuid, $path, $rev) = @_;
     my $myuuid = $repos->fs->get_uuid;
     return unless HAS_SVN_MIRROR && $uuid ne $myuuid;
-    my ($m, $mpath) = SVN::Mirror::has_local ($repos, "$uuid:$path");
+    my ($m, $mpath) = _has_local ($repos, "$uuid:$path");
     return ("$m->{target_path}$mpath",
 	    $rev ? $m->find_local_rev ($rev) : $rev) if $m;
 }
@@ -386,7 +386,7 @@ sub resolve_svm_source {
     my $myuuid = $repos->fs->get_uuid;
     return ($path) if ($uuid eq $myuuid);
     return unless HAS_SVN_MIRROR;
-    my ($m, $mpath) = SVN::Mirror::has_local ($repos, "$uuid:$path");
+    my ($m, $mpath) = _has_local ($repos, "$uuid:$path");
     return unless $m;
     return ("$m->{target_path}$mpath", $m);
 }
@@ -601,6 +601,9 @@ sub abs2rel {
         $rel = catdir($new_basedir, $rel);
     }
 
+    # resemble file::spec pre-3.13 behaviour, return empty string.
+    return '' if $rel eq '.';
+
     $rel =~ s/\Q$SEP/$sep/go if $sep and $SEP ne $sep;
     return $rel;
 }
@@ -693,7 +696,12 @@ sub make_path {
     return undef if !defined($path) or -d $path;
 
     require File::Path;
-    return File::Path::mkpath([$path]);
+    my @ret = eval { File::Path::mkpath([$path]) };
+    if ($@) {
+	$@ =~ s/ at .*//;
+	die $@;
+    }
+    return @ret;
 }
 
 =head3 splitpath ($path)
@@ -847,7 +855,9 @@ sub _copies_in_root {
     my ($root) = @_;
     my $copies;
     my $changed = $root->paths_changed;
+    my $pool = SVN::Pool->new_default;
     for (keys %$changed) {
+	$pool->clear;
 	next if $changed->{$_}->change_kind == $SVN::Fs::PathChange::delete;
 	my ($copyfrom_rev, $copyfrom_path) = $root->copied_from ($_);
 	$copies->{$_} = [$copyfrom_rev, $copyfrom_path]
@@ -883,6 +893,49 @@ sub find_prev_copy {
     return ($root, $copy);
 }
 
+
+# this is the cached and faster version of svn::mirror::has_local,
+# which should be deprecated eventually.
+my %mirror_cached;
+
+sub _list_mirror_cached {
+    my $repos = shift;
+    my $rev = $repos->fs->youngest_rev;
+    delete $mirror_cached{$repos}
+	unless ($mirror_cached{$repos}{rev} || -1) == $rev;
+    return %{$mirror_cached{$repos}{hash}}
+	if exists $mirror_cached{$repos};
+    my %mirrored = map {
+	my $m = SVN::Mirror->new( target_path => $_,
+				  repos => $repos,
+				  pool => SVN::Pool->new,
+				  get_source => 1 );
+	local $@;
+	eval { $m->init };
+	$@ ? () : ($_ => join(':', $m->{source_uuid}, $m->{source_path}))
+    } SVN::Mirror::list_mirror($repos);
+
+    $mirror_cached{$repos} = { rev => $rev, hash => \%mirrored};
+    return %mirrored;
+}
+
+sub _has_local {
+    my ($repos, $spec) = @_;
+    my %mirrored = _list_mirror_cached($repos);
+    while (my ($path, $mspec) = each(%mirrored)) {
+	my $mpath = $spec;
+	next unless $mpath =~ s/^\Q$mspec\E//;
+	my $m = SVN::Mirror->new (target_path => $path,
+				  repos => $repos,
+				  pool => SVN::Pool->new,
+				  get_source => 1);
+	$m->init;
+	$mpath = '' if $mpath eq '/';
+	return ($m, $mpath);
+    }
+    return;
+}
+
 1;
 
 __END__
